# $Id: komhtml.py,v 1.20 1996/07/17 12:37:19 ceder Exp $
# Copyright (C) 1996  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import cgi
import komconfig
import regsub
import string
import sys
import log

bad_auth='bad_auth'
not_a_komurl='not_a_komurl'

hostpart="http://" + komconfig.wwwhost
if komconfig.wwwport != 80:
    hostpart = hostpart + ":" + komconfig.wwwport

text_to_mark = None

def anchor(url, text, explanation = None):
    if explanation == None:
	return '<a href="' + url + '">' + text + '</a>'
    else:
	return ('<a href="' + url + '" onMouseOver="window.status='
		+ "'" + explanation + "'" + '; return true;">'
		+ text + '</a>')

def komurl(path, session = None):
    # Decide if session should be string or number, and make sure it always
    # is.
    if session:
	session = `session`
	base = hostpart + "/wwwkom/(" + session + ")/" + path
    else:
	base = hostpart + "/wwwkom/" + path

    if text_to_mark == None:
	return base

    if string.find(path, "?") == -1:
	return base + "?is_read=" + `text_to_mark`
    else:
	return base + "&is_read=" + `text_to_mark`


def komurl_dont_mark(path, session = None):
    if session != None:
	session = `session`
	return hostpart + "/wwwkom/(" + session + ")/" + path
    else:
	return hostpart + "/wwwkom/" + path


#
# Return a string which is a link to the conference CONF_NO using NAME
# as the text in the link.  
#
def conf_as_link(conf_no, name):
    return anchor(komurl('conf_stat?conf_no=' + `conf_no`), htmlquote(name))


def conf_as_direct_link(conf_no, name):
    return anchor(komurl('conf?conf_no=' + `conf_no`), htmlquote(name))


def text_as_link(text_no):
    return anchor(komurl('text?text_no=' + `text_no`), `text_no`)


def htmlquote(s):
    return cgi.escape(s)


class decodeurl:
    """Decoded WWW-kom URL.

    Usage:  x = decodeurl("/some/url")
    Throws not_a_komurl, or returns an object with the following members:
	pers_no:  a number
	cmd:      the command
	map:      any further arguments"""

    def __init__(self, url):
	# Assertion: url is on the format "/wwwkom/(4)/cmd?args"
	# or "/wwwkom/cmd?args" where the "?args" part is optional.
	u = string.split(url, '/')
	if len(u) < 3 or len(u[2]) < 1 or u[0] != '' or u[1] != 'wwwkom':
	    raise not_a_komurl, "Too short or bad: " + `u`
	u[0:2] = []
	if u[0][0] == '(' and u[0][-1] == ')':
	    if len(u[0]) < 3:
		raise not_a_komurl, "Empty session: " + `u[0]`
	    try:
		self.pers_no = string.atoi(u[0][1:-1])
	    except ValueError:
		raise not_a_komurl, "Bad session: " + `u[0]`
	    u[0:1] = []
	else:
	    self.pers_no = None
	if len(u) != 1:
	    raise not_a_komurl, "Expected a single command, got " + `u`
	arglist=string.split(u[0], "?")
	if len(arglist) == 1:
	    self.cmd = arglist[0]
	    self.rawarglist = ""
	    self.arglist = {}
	elif len(arglist) == 2:
	    self.cmd = arglist[0]
	    self.rawarglist = arglist[1]
	    self.arglist = form_data(arglist[1])
	else:
	    raise not_a_komurl, "Multiple question marks in arglist: " + u[0]

    def as_string(self, want_session = 0):
	return hostpart + self.as_string_sans_hostpart(want_session)

    def as_string_with_session(self):
	return self.as_string(1)

    def as_string_sans_hostpart(self, want_session = 0):
	res = "/wwwkom/"
	if self.pers_no and want_session:
	    res = res + "(" + `self.pers_no` + ")/"
	res = res + self.cmd
	if self.rawarglist:
	    res = res + "?" + self.rawarglist
	return res

    def as_string_sans_hostpart_with_session(self):
	return self.as_string_sans_hostpart(1)

    def getnum(self, key):
	try:
	    x = string.atoi(self.arglist[key])
	    log.debug("getnum(" + key + ") returns " + `x`)
	    return x
	except:
	    return None

    def getstr_empty(self, key):
	try:
	    return self.arglist[key]
	except KeyError:
	    return ""

    def getstr(self, key):
	try:
	    return self.arglist[key]
	except KeyError:
	    return None

def body_as_html(s):
    s = htmlquote(s)
    lines = []
    for line in string.split(s, '\n'):
	if len(line) < 50:
	    lines.append(line + '<br>')
	else:
	    lines.append(line)
    s = string.join(lines, '\n')
    gsub = regsub.gsub
    s = gsub('\n\n', '\n\n<p>', s)
    s = gsub('\(http://[^ "\n!,<>]*\)', '<a href="\\1">\\1</a>', s)
    s = gsub('\(ftp://[^ "\n!,<>]*\)', '<a href="\\1">\\1</a>', s)
    s = gsub('\(gopher://[^ "\n!,<>]*\)', '<a href="\\1">\\1</a>', s)
    s = gsub('\(file://[^ "\n!,<>]*\)', '<a href="\\1">\\1</a>', s)
    s = gsub('\(wais://[^ "\n!,<>]*\)', '<a href="\\1">\\1</a>', s)
    # The URL below will not work, but management told me to put it here.
    s = gsub('\(kom://[^ "\n!,<>]*\)', '<a href="\\1">\\1</a>', s)
    s = gsub('<br>\n*<p>', '\n\n<p>', s)
    s = gsub('\(^\|[\n (]\)\([0-9][0-9]*\)\([\n .]\|$\)',
	     '\\1<a href="text?text_no=\\2">\\2</a>\\3', s)
    return s


class form_data(cgi.SvFormContentDict):
    def __init__(self, arg):
	self.dict = cgi.parse_qs(arg)

def iconref(file, alt):
    return '<img src="/' + file + '" border=0 alt="' + alt + '">'

def icon(file, alt, link, explanation = None):
    return anchor(link, iconref(file, alt), explanation)
